import pytest
import snappi


def test_choice(api):
    """Test choice functionality

    Accessing a property that is a choice object or its child properties 
    should not change the choice.
    The only thing that should change the choice is when choice object's 
    child property is set.
    """
    config = api.config()
    config.devices.device(name='d1').device(name='d2')
    flow = config.flows.flow(name='f')[-1]
    flow.tx_rx.device.tx_names = [config.devices[0].name]
    flow.tx_rx.device.rx_names = [config.devices[1].name]
    flow.packet.ethernet().vlan().vlan().ipv4()
    assert (flow.packet[0].parent.choice == 'ethernet')
    assert (flow.packet[1].parent.choice == 'vlan')
    assert (flow.packet[2].parent.choice == 'vlan')
    assert (flow.packet[3].parent.choice == 'ipv4')

    eth = flow.packet[0]
    assert (eth.__class__ == snappi.FlowEthernet)
    eth_type = eth.ether_type
    assert (eth_type.__class__ == snappi.FlowPattern)
    eth.src.value = '00:00:01:00:00:01'
    assert (eth.src.choice == 'value')
    eth.src.values = ['00:00:01:00:00:01', '00:00:01:00:00:0a']
    assert (eth.src.choice == 'values')

    print(config)


if __name__ == '__main__':
    pytest.main(['-vv', '-s', __file__])



from flask import Flask, request, Response
import threading
import json
import snappi

app = Flask(__name__)
CONFIG = None


@app.route('/config', methods=['POST'])
def set_config():
    global CONFIG
    config = snappi.api().config()
    config.deserialize(request.data.decode('utf-8'))
    test = config.options.port_options.location_preemption
    if test is not None and isinstance(test, bool) is False:
        return Response(status=590,
                        response=json.dumps({'detail': 'invalid data type'}),
                        headers={'Content-Type': 'application/json'})
    else:
        CONFIG = config
        return Response(status=200)


@app.route('/config', methods=['GET'])
def get_config():
    global CONFIG
    return Response(CONFIG.serialize(),
                    mimetype='application/json',
                    status=200)


@app.route('/control/transmit', methods=['POST'])
def set_transmit_state():
    global CONFIG
    return Response(status=200)


@app.route('/results/port', methods=['POST'])
def get_port_metrics():
    global CONFIG
    api = snappi.api()
    port_metrics_request = api.port_metrics_request()
    port_metrics_request.deserialize(request.data.decode('utf-8'))
    port_metrics = api.port_metrics()
    for port in CONFIG.ports:
        port_metrics.metric(name=port.name, frames_tx=10000, frames_rx=10000)
    return Response(port_metrics.serialize(),
                    mimetype='application/json',
                    status=200)


@app.route('/results/flow', methods=['POST'])
def get_flow_metrics():
    global CONFIG
    api = snappi.api()
    flow_metrics_request = api.flow_metrics_request()
    flow_metrics_request.deserialize(request.data.decode('utf-8'))
    flow_metrics = api.flow_metrics()
    for flow in CONFIG.flows:
        flow_metrics.metric(name=flow.name, frames_tx=10000, frames_rx=10000)
    return Response(flow_metrics.serialize(),
                    mimetype='application/json',
                    status=200)


@app.after_request
def after_request(resp):
    print(request.method, request.url, ' -> ', resp.status)
    return resp


def web_server():
    app.run(port=80, debug=True, use_reloader=False)


class SnappiServer(object):
    def __init__(self):
        self._CONFIG = None

    def start(self):
        self._web_server_thread = threading.Thread(target=web_server)
        self._web_server_thread.setDaemon(True)
        self._web_server_thread.start()
        return self



from .snappi import Config
from .snappi import Port
from .snappi import PortList
from .snappi import Lag
from .snappi import LagProtocol
from .snappi import LagLacp
from .snappi import DevicePattern
from .snappi import DeviceCounter
from .snappi import LagStatic
from .snappi import DeviceEthernet
from .snappi import DeviceVlan
from .snappi import DeviceVlanList
from .snappi import DeviceIpv4
from .snappi import DeviceBgpv4
from .snappi import DeviceBgpv4RouteRange
from .snappi import DeviceBgpCommunity
from .snappi import DeviceBgpCommunityList
from .snappi import DeviceBgpAsPath
from .snappi import DeviceBgpAsPathSegment
from .snappi import DeviceBgpAsPathSegmentList
from .snappi import DeviceBgpv4RouteRangeList
from .snappi import DeviceBgpv6RouteRange
from .snappi import DeviceBgpv6RouteRangeList
from .snappi import DeviceIpv6
from .snappi import DeviceBgpv6
from .snappi import LagList
from .snappi import Layer1
from .snappi import Layer1AutoNegotiation
from .snappi import Layer1FlowControl
from .snappi import Layer1Ieee8021qbb
from .snappi import Layer1Ieee8023x
from .snappi import Layer1List
from .snappi import Capture
from .snappi import CaptureFilter
from .snappi import CaptureCustom
from .snappi import CaptureEthernet
from .snappi import CaptureField
from .snappi import CaptureVlan
from .snappi import CaptureIpv4
from .snappi import CaptureFilterList
from .snappi import CaptureList
from .snappi import Device
from .snappi import DeviceIpv4Loopback
from .snappi import DeviceIpv6Loopback
from .snappi import DeviceList
from .snappi import Flow
from .snappi import FlowTxRx
from .snappi import FlowPort
from .snappi import FlowDevice
from .snappi import FlowHeader
from .snappi import FlowCustom
from .snappi import FlowBitPattern
from .snappi import FlowBitList
from .snappi import FlowBitCounter
from .snappi import FlowBitPatternList
from .snappi import FlowEthernet
from .snappi import FlowPattern
from .snappi import FlowCounter
from .snappi import FlowVlan
from .snappi import FlowVxlan
from .snappi import FlowIpv4
from .snappi import FlowIpv4Priority
from .snappi import FlowIpv4Tos
from .snappi import FlowIpv4Dscp
from .snappi import FlowIpv6
from .snappi import FlowPfcPause
from .snappi import FlowEthernetPause
from .snappi import FlowTcp
from .snappi import FlowUdp
from .snappi import FlowGre
from .snappi import FlowGtpv1
from .snappi import FlowGtpExtension
from .snappi import FlowGtpExtensionList
from .snappi import FlowGtpv2
from .snappi import FlowArp
from .snappi import FlowHeaderList
from .snappi import FlowSize
from .snappi import FlowSizeIncrement
from .snappi import FlowSizeRandom
from .snappi import FlowRate
from .snappi import FlowDuration
from .snappi import FlowFixedPackets
from .snappi import FlowFixedSeconds
from .snappi import FlowBurst
from .snappi import FlowContinuous
from .snappi import FlowList
from .snappi import ConfigOptions
from .snappi import PortOptions
from .snappi import TransmitState
from .snappi import LinkState
from .snappi import CaptureState
from .snappi import StateMetrics
from .snappi import PortState
from .snappi import PortStateList
from .snappi import FlowState
from .snappi import FlowStateList
from .snappi import Capabilities
from .snappi import PortMetricsRequest
from .snappi import PortMetricList
from .snappi import PortMetric
from .snappi import CaptureRequest
from .snappi import FlowMetricsRequest
from .snappi import FlowMetricList
from .snappi import FlowMetric
from .snappi import FlowMetricGroup
from .snappi import FlowMetricGroupList
from .snappi import Bgpv4MetricsRequest
from .snappi import Bgpv4Metrics
from .snappi import Bgpv4Metric
from .snappi import Bgpv4MetricList
from .snappi import Api
from .snappi import HttpApi
from .snappi import api



import pytest


def test_layer1(api):
    """Test layer1
    """
    config = api.config()
    config.layer1.layer1()
    assert(len(config.layer1) == 1)


if __name__ == '__main__':
    pytest.main(['-vv', '-s', __file__])



import pytest


def test_port_metrics(api, b2b_config):
    """Get port metrics
    """
    api.set_config(b2b_config)
    port_metrics_request = api.port_metrics_request()
    port_metrics = api.get_port_metrics(port_metrics_request)
    for metric in port_metrics:
        print(metric)


if __name__ == '__main__':
    pytest.main(['-vv', '-s', __file__])



import pytest


def test_e2e_port_flow_config(api):
    """Demonstrates an end to end configuration
    """
    config = api.config()

    tx_port, rx_port = config.ports \
        .port(name='Tx Port', location='10.36.74.26;02;13') \
        .port(name='Rx Port', location='10.36.74.26;02;14')

    flow = config.flows.flow(name='Tx -> Rx Flow')[0]
    flow.tx_rx.port.tx_name = tx_port.name
    flow.tx_rx.port.rx_name = rx_port.name
    flow.size.fixed = 128
    flow.rate.pps = 1000
    flow.duration.fixed_packets.packets = 10000

    _, _, ip, _ = flow.packet.ethernet().vlan().ipv4().tcp()

    eth = flow.packet[0]
    eth.src.value = '00:00:01:00:00:01'
    eth.dst.values = ['00:00:02:00:00:01', '00:00:02:00:00:01']

    # primitive choice
    # @property
    # getter raises exception if choice is not valid
    # setter sets the choice or no setter at all?
    ip.src.value = '1.1.1.1'
    ip.src.values = ['1.1.1.1']
    # complex choice
    # @staticmethod returns instance of class, sets the choice
    # getter raises exception if choice is not valid
    # ip.src.Increment(start='', step='', count=1)
    ip.src.increment.start = '1.1.1.1'
    ip.src.increment.step = '0.0.0.1'
    ip.src.increment.count = 10
    print(ip)
    ip.dst.decrement.start = '1.1.2.200'
    ip.dst.decrement.step = '0.0.0.1'
    ip.dst.decrement.count = 10

    ip.priority.dscp.ecn.value = ip.priority.dscp.ECN_CAPABLE_TRANSPORT_1
    ip.priority.dscp.ecn.metric_group = 'ip.priority.dscp.ecn'

    # set and get the configuration
    api.set_config(config)
    print(api.get_config())

    # start transmit
    transmit_state = api.transmit_state()
    transmit_state.state = 'start'
    api.set_transmit_state(transmit_state)

    # get port metrics
    port_metrics_request = api.port_metrics_request()
    port_metrics = api.get_port_metrics(port_metrics_request)
    for metric in port_metrics:
        print(metric)

    # get flow metrics
    flow_metrics_request = api.flow_metrics_request()
    flow_metrics = api.get_flow_metrics(flow_metrics_request)
    for metric in flow_metrics:
        print(metric)


if __name__ == '__main__':
    pytest.main(['-vv', '-s', __file__])



import pytest


def test_bgp_as_path(api):
    """Test BGP AS Path functionality
    """
    config = api.config()
    p1, p2 = config.ports.port(name='p1').port(name='p2')
    d = config.devices.device(name='d', container_name=p1.name)[-1]

    # setup ethernet
    eth = d.ethernet
    eth.name = 'e'
    eth.mac.value = '00:01:00:00:00:01'

    # setup ipv4
    ip = eth.ipv4
    ip.name = 'i4'
    ip.address.value = '1.1.1.1'

    # setup bgp
    bgp = ip.bgpv4
    bgp.name = 'b4'

    # setup route range
    rr = bgp.bgpv4_route_ranges.bgpv4routerange()[-1]
    rr.range_count = 1
    rr.address_count = 1
    rr.address.value = '4.4.4.4'
    rr.address_step.value = '0.0.0.1'
    rr.prefix.value = '32'
    rr.as_path.as_set_mode = rr.as_path.INCLUDE_AS_SEQ  # one per route range
    seg = rr.as_path.as_path_segments.bgpaspathsegment()[-1]  
    seg.as_numbers = [1, 2, 3, 4, 5, 6]

    print(config)


if __name__ == '__main__':
    pytest.main(['-s', __file__])
